@@ -19,17 +19,15 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 
-""" Implements the `mkdocs` renderer. It uses the `markdown` renderer to
-produce an MkDocs-compatible folder structure. """
-
 from nr.databind.core import Field, Struct, ProxyType
 from nr.interface import implements, override
 from pydoc_markdown.contrib.renderers.markdown import MarkdownRenderer
-from pydoc_markdown.interfaces import Renderer, Resolver, Server
-from pydoc_markdown.reflection import Object, ModuleGraph
-from pydoc_markdown.util.pages import Page, Pages
+from pydoc_markdown.interfaces import Context, Renderer, Resolver, Server, Builder
+from pydoc_markdown.util.pages import Page
+from pydoc_markdown.util.knownfiles import KnownFiles
 from typing import Dict, Iterable, List, Optional, Tuple
 import copy
+import docspec
 import fnmatch
 import logging
 import os
@@ -47,22 +45,47 @@ class CustomizedMarkdownRenderer(MarkdownRenderer):
   render_toc = Field(bool, default=False)
 
 
-@implements(Renderer, Server)
+@implements(Renderer, Server, Builder)
 class MkdocsRenderer(Struct):
+  """
+  Produces Markdown files in a layout compatible with [MkDocs][0] and can be used with the
+  Pydoc-Markdown `--server` option for a live-preview. The `--bootstrap mkdocs` option can
+  be used to create a Pydoc-Markdown configuration file with the MkDocs template.
+
+  Example configuration:
+
+  ```yml
+  renderer:
+    type: mkdocs
+    pages:
+      - title: Home
+        name: index
+        source: README.md
+      - title: API Documentation
+        contents:
+          - '*'
+    mkdocs_config:
+      mkdocs_config:
+        site_name: My Project
+        theme: readthedocs
+  ```
+
+  ### Options
+  """
+
   #: The output directory for the generated Markdown files. Defaults to
   #: `build/docs`.
   output_directory = Field(str, default='build/docs')
 
-  #: Name of the content directory. Defaults to "content".
+  #: Name of the content directory (inside the #output_directory). Defaults to "content".
   content_directory_name = Field(str, default='content')
 
-  #: Delete the content directory in the #output_directory before rendering.
-  #: Note that this does not play well with pre-existing content, as it will
-  #: be deleted as well. Defaults to False.
-  clean_render = Field(bool, default=False)
+  #: Remove files generated in a previous pass by the Mkdocs renderer before rendering
+  #: again. Defaults to `True`.
+  clean_render = Field(bool, default=True)
 
   #: The pages to render into the output directory.
-  pages = Field(Pages)
+  pages = Field(Page.collection_type())
 
   #: Markdown renderer settings.
   markdown = Field(CustomizedMarkdownRenderer, default=CustomizedMarkdownRenderer)
@@ -75,6 +98,8 @@ class MkdocsRenderer(Struct):
   #: `mkdocs.yml` file.
   mkdocs_config = Field(dict, default=dict, nullable=True)
 
+  _context = Field(Context, default=None, hidden=True)  # Initialized in #init()
+
   @property
   def content_dir(self) -> str:
     return os.path.join(self.output_directory, self.content_directory_name)
@@ -89,6 +114,9 @@ class MkdocsRenderer(Struct):
           result.append({page.title: page.href})
         else:
           filename = os.path.relpath(page_to_filename[id(page)], self.content_dir)
+          if os.name == 'nt':
+            # Make generated configuration more portable across operating systems (see #129).
+            filename = filename.replace('\\', '/')
           result.append({page.title: filename})
       return result
     return _generate(self.pages)
@@ -96,41 +124,47 @@ class MkdocsRenderer(Struct):
   # Renderer
 
   @override
-  def render(self, graph: ModuleGraph) -> None:
-    if self.clean_render and os.path.isdir(self.content_dir):
-      logger.info('Cleaning directory "%s"', self.content_dir)
-      shutil.rmtree(self.content_dir)
+  def render(self, modules: List[docspec.Module]) -> None:
+    known_files = KnownFiles(self.output_directory)
+    if self.clean_render:
+      for file_ in known_files.load():
+        try:
+          os.remove(file_.name)
+        except FileNotFoundError:
+          pass
 
     page_to_filename = {}
 
-    for item in self.pages.iter_hierarchy():
-      filename = item.filename(self.content_dir, '.md')
-      if not filename:
-        continue
+    with known_files:
+      for item in self.pages.iter_hierarchy():
+        filename = item.filename(self.content_dir, '.md')
+        page_to_filename[id(item.page)] = filename
+        if not item.page.has_content():
+          continue
 
-      page_to_filename[id(item.page)] = filename
-      self.markdown.filename = filename
-      item.page.render(filename, graph, self.markdown)
+        self.markdown.filename = filename
+        item.page.render(filename, modules, self.markdown, context_directory=self._context.directory)
+        known_files.append(filename)
 
-    config = copy.deepcopy(self.mkdocs_config)
-    if self.site_name:
-      config['site_name'] = self.site_name
-    if not config.get('site_name'):
-      config['site_name'] = 'My Project'
-    config['docs_dir'] = self.content_directory_name
-    config['nav'] = self.generate_mkdocs_nav(page_to_filename)
+      config = copy.deepcopy(self.mkdocs_config)
+      if self.site_name:
+        config['site_name'] = self.site_name
+      if not config.get('site_name'):
+        config['site_name'] = 'My Project'
+      config['docs_dir'] = self.content_directory_name
+      config['nav'] = self.generate_mkdocs_nav(page_to_filename)
 
-    if self.mkdocs_config is not None:
-      filename = os.path.join(self.output_directory, 'mkdocs.yml')
-      logger.info('Rendering "%s"', filename)
-      with open(filename, 'w') as fp:
-        yaml.dump(config, fp)
+      if self.mkdocs_config is not None:
+        filename = os.path.join(self.output_directory, 'mkdocs.yml')
+        logger.info('Rendering "%s"', filename)
+        with known_files.open(filename, 'w') as fp:
+          yaml.dump(config, fp)
 
   @override
-  def get_resolver(self, graph: ModuleGraph) -> Optional[Resolver]:
+  def get_resolver(self, modules: List[docspec.Module]) -> Optional[Resolver]:
     # TODO (@NiklasRosenstein): The resolver returned by the Markdown
     #   renderer does not implement linking across multiple pages.
-    return self.markdown.get_resolver(graph)
+    return self.markdown.get_resolver(modules)
 
   # Server
 
@@ -147,3 +181,17 @@ class MkdocsRenderer(Struct):
     # While MkDocs does support file watching and automatic reloading,
     # it appears to be a bit quirky. Let's just restart the whole process.
     process.terminate()
+
+  # Builder
+
+  @override
+  def build(self, site_dir: str=None) -> None:
+    command = ['mkdocs', 'build', '--clean', '--site-dir', site_dir]
+    subprocess.check_call(command, cwd=self.output_directory)
+
+  # PluginBase
+
+  @override
+  def init(self, context: Context) -> None:
+    self._context = context
+    self.markdown.init(context)
