@@ -27,10 +27,10 @@ to document.
 
 from nr.databind.core import Collection, Field, ProxyType, Struct
 from pydoc_markdown.interfaces import Renderer
-from pydoc_markdown.reflection import ModuleGraph
 from typing import Iterable, Optional, List
 import collections
 import copy
+import docspec
 import fnmatch
 import logging
 import os
@@ -62,6 +62,13 @@ class IterHierarchyItem(Struct):
     return filename
 
 
+class PageCollectionMixin(list):
+
+  def iter_hierarchy(self) -> Iterable[IterHierarchyItem]:
+    for page in self:
+      yield from page.iter_hierarchy()
+
+
 @Page.implementation  # pylint: disable=function-redefined
 class Page(Struct):
   """
@@ -98,6 +105,9 @@ class Page(Struct):
     if not self.name:
       self.name = re.sub(r'\s+', '-', self.title.lower())
 
+  def has_content(self) -> bool:
+    return bool(self.source or self.contents)
+
   def iter_hierarchy(self, parent_chain: List['Page'] = None) -> Iterable[IterHierarchyItem]:
     if parent_chain is None:
       parent_chain = []
@@ -105,56 +115,65 @@ class Page(Struct):
     for child in self.children:
       yield from child.iter_hierarchy(parent_chain + [self])
 
-  def filtered_graph(self, graph: ModuleGraph) -> ModuleGraph:
+  def filtered_modules(self, modules: List[docspec.Module]) -> List[docspec.Module]:
     """
     Creates a copy of the module graph where only the API objects selected
     via #Page.contents are visible.
     """
 
-    def _match(node):
+    modules = copy.deepcopy(modules)
+    reverse_map = docspec.ReverseMap(modules)
+    matched_contents = set()
+
+    def _match(obj: docspec.ApiObject) -> bool:
+      if getattr(obj, 'members', []):
+        return True
       if self.contents:
-        path = node.path()
-        return any(fnmatch.fnmatch(path, x) for x in self.contents or ())
+        path = '.'.join(x.name for x in reverse_map.path(obj))
+        for x in self.contents:
+          if fnmatch.fnmatch(path, x):
+            matched_contents.add(x)
+            return True
       return False
 
-    def _update_nodes(x):
-      x.visible = x.visible and _match(x)
-      # Make sure all parents are visible as well.
-      while x and x.visible:
-        x = x.parent
-        if x:
-          x.visible = True
+    docspec.filter_visit(modules, _match, order='post')
 
-    clone = copy.deepcopy(graph)
-    clone.visit(_update_nodes)
-    return clone
+    unmatched_contents = set(self.contents or ()) - matched_contents
+    if unmatched_contents:
+      logger.warning(
+        'Page(title=%r).contents has unmatched elements: %s. Did you spell it correctly? Does '
+          'a processor filter out this object?',
+        self.title,
+        ', '.join(unmatched_contents),
+      )
+
+    return modules
 
   def render(
       self,
       filename: str,
-      graph: ModuleGraph,
-      renderer: Renderer
+      modules: List[docspec.ApiObject],
+      renderer: Renderer,
+      context_directory: str,
       ) -> None:
     """
-    Renders the page by either copying the *source* to the specified
-    *filename* or by rendering the *contents* from the *graph* using the
-    specified *renderer*.
+    Renders the page by either copying the *source* to the specified *filename* or by
+    rendering the *contents* from the *modules* using the specified *renderer*.
 
     Note that the *renderer* should be pre-configured to output to *filename*.
     """
 
     os.makedirs(os.path.dirname(filename), exist_ok=True)
     if self.source:
-      logger.info('Writing "%s" (source: "%s")', filename, self.source)
-      shutil.copyfile(self.source, filename)
+      logger.info('Writing "%s" (source: "%s")', filename, os.path.join(context_directory, self.source))
+      shutil.copyfile(os.path.join(context_directory, self.source), filename)
     else:
       logger.info('Rendering "%s"', filename)
-      renderer.render(self.filtered_graph(graph))
+      renderer.render(self.filtered_modules(modules))
 
-
-class Pages(Collection, list):
-  item_type = Page
-
-  def iter_hierarchy(self) -> Iterable[IterHierarchyItem]:
-    for page in self:
-      yield from page.iter_hierarchy()
+  @classmethod
+  def collection_type(cls) -> PageCollectionMixin:
+    return type(
+      '{}Collection'.format(cls.__name__),
+      (Collection, PageCollectionMixin),
+      {'item_type': cls})
